<?php

namespace Dipendra\Programming\Http\Controllers\Api;

use App\Http\Controllers\Api\UserController as BaseController;
use Dipendra\Programming\Http\Requests\ChapterRequest;
use Dipendra\Programming\Interfaces\ChapterRepositoryInterface;
use Dipendra\Programming\Models\Chapter;

/**
 * User API controller class.
 */
class ChapterUserController extends BaseController
{
    /**
     * Initialize chapter controller.
     *
     * @param type ChapterRepositoryInterface $chapter
     *
     * @return type
     */
    public function __construct(ChapterRepositoryInterface $chapter)
    {
        $this->repository = $chapter;
        $this->repository
                ->pushCriteria(app('Litepie\Repository\Criteria\RequestCriteria'))
                ->pushCriteria(new \Dipendra\Programming\Repositories\Criteria\ChapterUserCriteria());
        parent::__construct();
    }

    /**
     * Display a list of chapter.
     *
     * @return json
     */
    public function index(ChapterRequest $request)
    {
        $chapters  = $this->repository
            ->setPresenter('\\Dipendra\\Programming\\Repositories\\Presenter\\ChapterListPresenter')
            ->scopeQuery(function($query){
                return $query->orderBy('id','DESC');
            })->all();
        $chapters['code'] = 2000;
        return response()->json($chapters) 
            ->setStatusCode(200, 'INDEX_SUCCESS');

    }

    /**
     * Display chapter.
     *
     * @param Request $request
     * @param Model   Chapter
     *
     * @return Json
     */
    public function show(ChapterRequest $request, Chapter $chapter)
    {

        if ($chapter->exists) {
            $chapter         = $chapter->presenter();
            $chapter['code'] = 2001;
            return response()->json($chapter)
                ->setStatusCode(200, 'SHOW_SUCCESS');;
        } else {
            return response()->json([])
                ->setStatusCode(400, 'SHOW_ERROR');
        }

    }

    /**
     * Show the form for creating a new chapter.
     *
     * @param Request $request
     *
     * @return json
     */
    public function create(ChapterRequest $request, Chapter $chapter)
    {
        $chapter         = $chapter->presenter();
        $chapter['code'] = 2002;
        return response()->json($chapter)
            ->setStatusCode(200, 'CREATE_SUCCESS');
    }

    /**
     * Create new chapter.
     *
     * @param Request $request
     *
     * @return json
     */
    public function store(ChapterRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.api');
            $chapter          = $this->repository->create($attributes);
            $chapter          = $chapter->presenter();
            $chapter['code']  = 2004;

            return response()->json($chapter)
                ->setStatusCode(201, 'STORE_SUCCESS');
        } catch (Exception $e) {
            return response()->json([
                'message' => $e->getMessage(),
                'code'    => 4004,
            ])->setStatusCode(400, 'STORE_ERROR');
        }

    }

    /**
     * Show chapter for editing.
     *
     * @param Request $request
     * @param Model   $chapter
     *
     * @return json
     */
    public function edit(ChapterRequest $request, Chapter $chapter)
    {
        if ($chapter->exists) {
            $chapter         = $chapter->presenter();
            $chapter['code'] = 2003;
            return response()-> json($chapter)
                ->setStatusCode(200, 'EDIT_SUCCESS');;
        } else {
            return response()->json([])
                ->setStatusCode(400, 'SHOW_ERROR');
        }
    }

    /**
     * Update the chapter.
     *
     * @param Request $request
     * @param Model   $chapter
     *
     * @return json
     */
    public function update(ChapterRequest $request, Chapter $chapter)
    {
        try {

            $attributes = $request->all();

            $chapter->update($attributes);
            $chapter         = $chapter->presenter();
            $chapter['code'] = 2005;

            return response()->json($chapter)
                ->setStatusCode(201, 'UPDATE_SUCCESS');


        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4005,
            ])->setStatusCode(400, 'UPDATE_ERROR');

        }
    }

    /**
     * Remove the chapter.
     *
     * @param Request $request
     * @param Model   $chapter
     *
     * @return json
     */
    public function destroy(ChapterRequest $request, Chapter $chapter)
    {

        try {

            $t = $chapter->delete();

            return response()->json([
                'message'  => trans('messages.success.delete', ['Module' => trans('programming::chapter.name')]),
                'code'     => 2006
            ])->setStatusCode(202, 'DESTROY_SUCCESS');

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4006,
            ])->setStatusCode(400, 'DESTROY_ERROR');
        }
    }
}
