<?php

namespace Dipendra\Programming\Http\Controllers\Api;

use App\Http\Controllers\Api\UserController as BaseController;
use Dipendra\Programming\Http\Requests\TutorialRequest;
use Dipendra\Programming\Interfaces\TutorialRepositoryInterface;
use Dipendra\Programming\Models\Tutorial;

/**
 * User API controller class.
 */
class TutorialUserController extends BaseController
{
    /**
     * Initialize tutorial controller.
     *
     * @param type TutorialRepositoryInterface $tutorial
     *
     * @return type
     */
    public function __construct(TutorialRepositoryInterface $tutorial)
    {
        $this->repository = $tutorial;
        $this->repository
                ->pushCriteria(app('Litepie\Repository\Criteria\RequestCriteria'))
                ->pushCriteria(new \Dipendra\Programming\Repositories\Criteria\TutorialUserCriteria());
        parent::__construct();
    }

    /**
     * Display a list of tutorial.
     *
     * @return json
     */
    public function index(TutorialRequest $request)
    {
        $tutorials  = $this->repository
            ->setPresenter('\\Dipendra\\Programming\\Repositories\\Presenter\\TutorialListPresenter')
            ->scopeQuery(function($query){
                return $query->orderBy('id','DESC');
            })->all();
        $tutorials['code'] = 2000;
        return response()->json($tutorials) 
            ->setStatusCode(200, 'INDEX_SUCCESS');

    }

    /**
     * Display tutorial.
     *
     * @param Request $request
     * @param Model   Tutorial
     *
     * @return Json
     */
    public function show(TutorialRequest $request, Tutorial $tutorial)
    {

        if ($tutorial->exists) {
            $tutorial         = $tutorial->presenter();
            $tutorial['code'] = 2001;
            return response()->json($tutorial)
                ->setStatusCode(200, 'SHOW_SUCCESS');;
        } else {
            return response()->json([])
                ->setStatusCode(400, 'SHOW_ERROR');
        }

    }

    /**
     * Show the form for creating a new tutorial.
     *
     * @param Request $request
     *
     * @return json
     */
    public function create(TutorialRequest $request, Tutorial $tutorial)
    {
        $tutorial         = $tutorial->presenter();
        $tutorial['code'] = 2002;
        return response()->json($tutorial)
            ->setStatusCode(200, 'CREATE_SUCCESS');
    }

    /**
     * Create new tutorial.
     *
     * @param Request $request
     *
     * @return json
     */
    public function store(TutorialRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.api');
            $tutorial          = $this->repository->create($attributes);
            $tutorial          = $tutorial->presenter();
            $tutorial['code']  = 2004;

            return response()->json($tutorial)
                ->setStatusCode(201, 'STORE_SUCCESS');
        } catch (Exception $e) {
            return response()->json([
                'message' => $e->getMessage(),
                'code'    => 4004,
            ])->setStatusCode(400, 'STORE_ERROR');
        }

    }

    /**
     * Show tutorial for editing.
     *
     * @param Request $request
     * @param Model   $tutorial
     *
     * @return json
     */
    public function edit(TutorialRequest $request, Tutorial $tutorial)
    {
        if ($tutorial->exists) {
            $tutorial         = $tutorial->presenter();
            $tutorial['code'] = 2003;
            return response()-> json($tutorial)
                ->setStatusCode(200, 'EDIT_SUCCESS');;
        } else {
            return response()->json([])
                ->setStatusCode(400, 'SHOW_ERROR');
        }
    }

    /**
     * Update the tutorial.
     *
     * @param Request $request
     * @param Model   $tutorial
     *
     * @return json
     */
    public function update(TutorialRequest $request, Tutorial $tutorial)
    {
        try {

            $attributes = $request->all();

            $tutorial->update($attributes);
            $tutorial         = $tutorial->presenter();
            $tutorial['code'] = 2005;

            return response()->json($tutorial)
                ->setStatusCode(201, 'UPDATE_SUCCESS');


        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4005,
            ])->setStatusCode(400, 'UPDATE_ERROR');

        }
    }

    /**
     * Remove the tutorial.
     *
     * @param Request $request
     * @param Model   $tutorial
     *
     * @return json
     */
    public function destroy(TutorialRequest $request, Tutorial $tutorial)
    {

        try {

            $t = $tutorial->delete();

            return response()->json([
                'message'  => trans('messages.success.delete', ['Module' => trans('programming::tutorial.name')]),
                'code'     => 2006
            ])->setStatusCode(202, 'DESTROY_SUCCESS');

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4006,
            ])->setStatusCode(400, 'DESTROY_ERROR');
        }
    }
}
