<?php

namespace Dipendra\Programming\Http\Controllers;

use App\Http\Controllers\AdminController as BaseController;
use Form;
use Dipendra\Programming\Http\Requests\ChapterRequest;
use Dipendra\Programming\Interfaces\ChapterRepositoryInterface;
use Dipendra\Programming\Models\Chapter;

/**
 * Admin web controller class.
 */
class ChapterAdminController extends BaseController
{
    // use ChapterWorkflow;
    /**
     * Initialize chapter controller.
     *
     * @param type ChapterRepositoryInterface $chapter
     *
     * @return type
     */
    public function __construct(ChapterRepositoryInterface $chapter)
    {
        $this->repository = $chapter;
        parent::__construct();
    }

    /**
     * Display a list of chapter.
     *
     * @return Response
     */
    public function index(ChapterRequest $request)
    {
        if ($request->wantsJson()) {
            return $this->getJson($request);
        }
        $this   ->theme->prependTitle(trans('programming::chapter.names').' :: ');
        return $this->theme->of('programming::admin.chapter.index')->render();
    }

    /**
     * Display a list of chapter.
     *
     * @return Response
     */
    public function getJson(ChapterRequest $request)
    {
        $pageLimit = $request->input('pageLimit');

        $chapters  = $this->repository
                ->pushCriteria(app('Litepie\Repository\Criteria\RequestCriteria'))
                ->setPresenter('\\Dipendra\\Programming\\Repositories\\Presenter\\ChapterListPresenter')
                ->scopeQuery(function($query){
                    return $query->orderBy('id','DESC');
                })->paginate($pageLimit);
        $chapters['recordsTotal']    = $chapters['meta']['pagination']['total'];
        $chapters['recordsFiltered'] = $chapters['meta']['pagination']['total'];
        $chapters['request']         = $request->all();
        return response()->json($chapters, 200);

    }

    /**
     * Display chapter.
     *
     * @param Request $request
     * @param Model   $chapter
     *
     * @return Response
     */
    public function show(ChapterRequest $request, Chapter $chapter)
    {
        if (!$chapter->exists) {
            return response()->view('programming::admin.chapter.new', compact('chapter'));
        }

        Form::populate($chapter);
        return response()->view('programming::admin.chapter.show', compact('chapter'));
    }

    /**
     * Show the form for creating a new chapter.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ChapterRequest $request)
    {

        $chapter = $this->repository->newInstance([]);

        Form::populate($chapter);

        return response()->view('programming::admin.chapter.create', compact('chapter'));

    }

    /**
     * Create new chapter.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ChapterRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.web');
            $chapter          = $this->repository->create($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('programming::chapter.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/programming/chapter/'.$chapter->getRouteKey())
            ], 201);


        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
            ], 400);
        }
    }

    /**
     * Show chapter for editing.
     *
     * @param Request $request
     * @param Model   $chapter
     *
     * @return Response
     */
    public function edit(ChapterRequest $request, Chapter $chapter)
    {
        Form::populate($chapter);
        return  response()->view('programming::admin.chapter.edit', compact('chapter'));
    }

    /**
     * Update the chapter.
     *
     * @param Request $request
     * @param Model   $chapter
     *
     * @return Response
     */
    public function update(ChapterRequest $request, Chapter $chapter)
    {
        try {

            $attributes = $request->all();

            $chapter->update($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('programming::chapter.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/programming/chapter/'.$chapter->getRouteKey())
            ], 201);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/programming/chapter/'.$chapter->getRouteKey()),
            ], 400);

        }
    }

    /**
     * Remove the chapter.
     *
     * @param Model   $chapter
     *
     * @return Response
     */
    public function destroy(ChapterRequest $request, Chapter $chapter)
    {

        try {

            $t = $chapter->delete();

            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('programming::chapter.name')]),
                'code'     => 202,
                'redirect' => trans_url('/admin/programming/chapter/0'),
            ], 202);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/programming/chapter/'.$chapter->getRouteKey()),
            ], 400);
        }
    }

}
