<?php

namespace Dipendra\Programming\Http\Controllers;

use App\Http\Controllers\AdminController as BaseController;
use Form;
use Dipendra\Programming\Http\Requests\TutorialRequest;
use Dipendra\Programming\Interfaces\TutorialRepositoryInterface;
use Dipendra\Programming\Models\Tutorial;

/**
 * Admin web controller class.
 */
class TutorialAdminController extends BaseController
{
    // use TutorialWorkflow;
    /**
     * Initialize tutorial controller.
     *
     * @param type TutorialRepositoryInterface $tutorial
     *
     * @return type
     */
    public function __construct(TutorialRepositoryInterface $tutorial)
    {
        $this->repository = $tutorial;
        parent::__construct();
    }

    /**
     * Display a list of tutorial.
     *
     * @return Response
     */
    public function index(TutorialRequest $request)
    {
        if ($request->wantsJson()) {
            return $this->getJson($request);
        }
        $this   ->theme->prependTitle(trans('programming::tutorial.names').' :: ');
        return $this->theme->of('programming::admin.tutorial.index')->render();
    }

    /**
     * Display a list of tutorial.
     *
     * @return Response
     */
    public function getJson(TutorialRequest $request)
    {
        $pageLimit = $request->input('pageLimit');

        $tutorials  = $this->repository
                ->pushCriteria(app('Litepie\Repository\Criteria\RequestCriteria'))
                ->setPresenter('\\Dipendra\\Programming\\Repositories\\Presenter\\TutorialListPresenter')
                ->scopeQuery(function($query){
                    return $query->orderBy('id','DESC');
                })->paginate($pageLimit);
        $tutorials['recordsTotal']    = $tutorials['meta']['pagination']['total'];
        $tutorials['recordsFiltered'] = $tutorials['meta']['pagination']['total'];
        $tutorials['request']         = $request->all();
        return response()->json($tutorials, 200);

    }

    /**
     * Display tutorial.
     *
     * @param Request $request
     * @param Model   $tutorial
     *
     * @return Response
     */
    public function show(TutorialRequest $request, Tutorial $tutorial)
    {
        if (!$tutorial->exists) {
            return response()->view('programming::admin.tutorial.new', compact('tutorial'));
        }

        Form::populate($tutorial);
        return response()->view('programming::admin.tutorial.show', compact('tutorial'));
    }

    /**
     * Show the form for creating a new tutorial.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TutorialRequest $request)
    {

        $tutorial = $this->repository->newInstance([]);

        Form::populate($tutorial);

        return response()->view('programming::admin.tutorial.create', compact('tutorial'));

    }

    /**
     * Create new tutorial.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TutorialRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.web');
            $tutorial          = $this->repository->create($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('programming::tutorial.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/programming/tutorial/'.$tutorial->getRouteKey())
            ], 201);


        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
            ], 400);
        }
    }

    /**
     * Show tutorial for editing.
     *
     * @param Request $request
     * @param Model   $tutorial
     *
     * @return Response
     */
    public function edit(TutorialRequest $request, Tutorial $tutorial)
    {
        Form::populate($tutorial);
        return  response()->view('programming::admin.tutorial.edit', compact('tutorial'));
    }

    /**
     * Update the tutorial.
     *
     * @param Request $request
     * @param Model   $tutorial
     *
     * @return Response
     */
    public function update(TutorialRequest $request, Tutorial $tutorial)
    {
        try {

            $attributes = $request->all();

            $tutorial->update($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('programming::tutorial.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/programming/tutorial/'.$tutorial->getRouteKey())
            ], 201);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/programming/tutorial/'.$tutorial->getRouteKey()),
            ], 400);

        }
    }

    /**
     * Remove the tutorial.
     *
     * @param Model   $tutorial
     *
     * @return Response
     */
    public function destroy(TutorialRequest $request, Tutorial $tutorial)
    {

        try {

            $t = $tutorial->delete();

            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('programming::tutorial.name')]),
                'code'     => 202,
                'redirect' => trans_url('/admin/programming/tutorial/0'),
            ], 202);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/programming/tutorial/'.$tutorial->getRouteKey()),
            ], 400);
        }
    }

}
